/* ====================================================================
 * Copyright (c) 2011 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#include "../ssl/ssl_locl.h"


#ifdef OPENSSL_FIPS

#define TLS1_PRF_DGST_SHIFT 10
#define SSL_HANDSHAKE_MAC_SHA256 0x80
#define TLS1_PRF_SHA256 (SSL_HANDSHAKE_MAC_SHA256 << TLS1_PRF_DGST_SHIFT)
#define SSL_MD_SHA256_IDX 4
#define SSL_MD_NUM_IDX 6

extern const EVP_MD *ssl_digest_methods[SSL_MD_NUM_IDX];
extern int ssl_mac_secret_size[SSL_MD_NUM_IDX];

int private_tls1_PRF(long digest_mask,
                     const void *seed1, int seed1_len,
                     const void *seed2, int seed2_len,
                     const void *seed3, int seed3_len,
                     const void *seed4, int seed4_len,
                     const void *seed5, int seed5_len,
                     const unsigned char *sec, int slen,
                     unsigned char *out1, unsigned char *out2, int olen);


int FIPS_selftest_tls(void)
{

  ssl_digest_methods[SSL_MD_SHA256_IDX] = EVP_sha256();
  ssl_mac_secret_size[SSL_MD_SHA256_IDX] =
    EVP_MD_size(ssl_digest_methods[SSL_MD_SHA256_IDX]);

  const char *seed = "seed";
  const char *secret = "secret";
  unsigned char out[16], dummy[16];
  const unsigned char expected[sizeof(out)] = {
    0x8e, 0x4d, 0x93, 0x25, 0x30, 0xd7, 0x65, 0xa0,
    0xaa, 0xe9, 0x74, 0xc3, 0x04, 0x73, 0x5e, 0xcc
  };
  long digest_mask = TLS1_PRF_SHA256;
  int ret = 1;                  /* success */

  private_tls1_PRF(digest_mask,
  	   seed, strlen(seed),
  	   NULL,       0,
  	   NULL,       0,
  	   NULL,       0,
  	   NULL,       0,
  	   secret, strlen(secret),
  	   out, dummy, sizeof(out));

  if ( memcmp(out, expected, sizeof(expected))) {
    ret = 0;                  /* failure */
  }
  return ret;
}


#endif
