/* ====================================================================
 * Copyright (c) 2004 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/*---------------------------------------------
  TLS Algorithm Validation Suite
  Test Program

  Donated to OpenSSL by:
  SUSE LLC
  ----------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include <ctype.h>
#include <openssl/ssl.h>

#include <openssl/err.h>
#include "e_os.h"

extern int private_tls1_PRF(long digest_mask,
	const void *seed1, int seed1_len,
	const void *seed2, int seed2_len,
	const void *seed3, int seed3_len,
	const void *seed4, int seed4_len,
	const void *seed5, int seed5_len,
	const unsigned char *sec, int slen,
	unsigned char *out1,
	unsigned char *out2, int olen);

#define SSL_HANDSHAKE_MAC_MD5 0x10
#define SSL_HANDSHAKE_MAC_SHA 0x20
#define SSL_HANDSHAKE_MAC_SHA256 0x80
#define SSL_HANDSHAKE_MAC_SHA384 0x100
#define TLS1_PRF_DGST_MASK   (0xff << TLS1_PRF_DGST_SHIFT)
#define TLS1_PRF_DGST_SHIFT 10
#define TLS1_PRF_MD5 (SSL_HANDSHAKE_MAC_MD5 << TLS1_PRF_DGST_SHIFT)
#define TLS1_PRF_SHA1 (SSL_HANDSHAKE_MAC_SHA << TLS1_PRF_DGST_SHIFT)
#define TLS1_PRF_SHA256 (SSL_HANDSHAKE_MAC_SHA256 << TLS1_PRF_DGST_SHIFT)
#define TLS1_PRF_SHA384 (SSL_HANDSHAKE_MAC_SHA384 << TLS1_PRF_DGST_SHIFT)
#define TLS1_PRF (TLS1_PRF_MD5 | TLS1_PRF_SHA1)

#ifndef OPENSSL_FIPS

int main(int argc, char *argv[])
{
    printf("No FIPS TLS support\n");
    return(0);
}

#else

#include <openssl/fips.h>
#include "fips_utl.h"

#define VERBOSE 0

/*-----------------------------------------------*/


/*================================================*/
/*----------------------------
  # CAVS 12.0
  # 'TLS' information for abc
  # Cases tested: [TLS 1.0/1.1] [TLS 1.2, SHA-256], [TLS 1.2, SHA-384], [TLS 1.2, SHA-512]
  # Generated on: Fri Aug 30 04:07:22 PM
  ----------------------------*/

int proc_file(char *rqfile, char *rspfile)
    {
    char afn[256], rfn[256];
    FILE *afp = NULL, *rfp = NULL;
    char ibuf[2048];
    char tbuf[2048];
    int len;
    char amode[8] = "";
    char atest[8] = "";
    int akeysz = 0;

    /* TLS stuff */
    int shabits = 0, premasterlength = 0, tls = 0, keyblocklength = 0;
    unsigned char *premastersecret = NULL;
    unsigned char serverHello_random[2048];
    unsigned int serverhellorandomlen = 0;
    unsigned char clientHello_random[2048];
    unsigned int clienthellorandomlen = 0;
    unsigned char server_random[2048];
    unsigned int serverrandomlen = 0;
    unsigned char client_random[2048];
    unsigned int clientrandomlen = 0;
    unsigned char mastersecret[2048];
    unsigned int digest_mask = 0;
    unsigned char seed[2048];
    unsigned char *keyblock = NULL;
    unsigned char *keyblocktmp = NULL;
    unsigned char tmpsecret[80];
    /* end of TLS stuff */

    int err = 0, step = 0;
    char *rp;
    EVP_CIPHER_CTX ctx;
    EVP_CIPHER_CTX_init(&ctx);

    if (!rqfile || !(*rqfile))
	{
	printf("No req file\n");
	return -1;
	}
    strcpy(afn, rqfile);

    if ((afp = fopen(afn, "r")) == NULL)
	{
	printf("Cannot open file: %s, %s\n",
	       afn, strerror(errno));
	return -1;
	}
    if (!rspfile)
	{
	strcpy(rfn,afn);
	rp=strstr(rfn,"req/");
#ifdef OPENSSL_SYS_WIN32
	if (!rp)
	    rp=strstr(rfn,"req\\");
#endif
	assert(rp);
	memcpy(rp,"rsp",3);
	rp = strstr(rfn, ".req");
	memcpy(rp, ".rsp", 4);
	rspfile = rfn;
	}
    if ((rfp = fopen(rspfile, "w")) == NULL)
	{
	printf("Cannot open file: %s, %s\n",
	       rfn, strerror(errno));
	fclose(afp);
	afp = NULL;
	return -1;
	}


    /* hey ho lets go */

    while (!err && (fgets(ibuf, sizeof(ibuf), afp)) != NULL)
	{
	tidy_line(tbuf, ibuf);
	printf("step=%d ibuf=%s",step,ibuf);
	switch (step)
	    {
	case 0:  /* read preamble */
	    if (ibuf[0] == '\n')
		{ /* end of preamble */
		/* not really anything mandatory, would need to check here */
		    {
		    fputs(ibuf, rfp);
		    ++ step;
		    }
		}
	    else if (ibuf[0] != '#')
		{
		printf("Invalid preamble item: %s\n", ibuf);
		err = 1;
		}
	    else
		{ /* process preamble */
		char *pp = ibuf+2;
		if (akeysz)
		    { /* insert current time & date */
		    time_t rtim = time(0);
		    fprintf(rfp, "# %s", ctime(&rtim));
		    }
		else
		    {
		    fputs(ibuf, rfp);
		    if (strncmp(pp, "'TLS' ", 5) == 0)
			{
			if (VERBOSE)
				printf("Test = %s, Mode = %s\n", atest, amode);
			}
		    else if (strncasecmp(pp, "Cases Tested: ", 14) == 0)
			{
			if (VERBOSE)
				printf("Cases %s\n", pp+14);
			}
		    }
		}
	    break;

	case 1:  /* [ENCRYPT] | [DECRYPT] */
	    if (ibuf[0] == '[')
		{
		fputs(ibuf, rfp);
		if (strncasecmp(ibuf, "[TLS 1.0/1.1]", 13 ) == 0) {
			tls = 10;
			shabits = 1;
			printf("TLS 1.0/1.1\n");
			premasterlength = keyblocklength = 0;
                } else
		if (sscanf(ibuf, "[TLS 1.2, SHA-%d]", &shabits) == 1) {
			tls = 12;
			printf("TLS 1.2 shabits %d\n", shabits);
			premasterlength = keyblocklength = 0;
                } else if (sscanf(ibuf, "[pre-master secret length = %d]", &premasterlength) == 1) {
			printf("premaster length %d\n", premasterlength);
			if (premasterlength % 8) {
				printf("premaster length not byte sized?? %d\n", premasterlength);
				err = 1;
				break;
			}
			premasterlength /= 8;
			free(premastersecret);
			premastersecret = malloc(premasterlength);
                } else if (sscanf(ibuf, "[key block length = %d]", &keyblocklength) == 1) {
			printf("key block length %d\n", keyblocklength);
			if (keyblocklength % 8) {
				printf("keyblock length not byte sized?? %d\n", keyblocklength);
				err = 1;
				break;
			}
			free(keyblock);
			free(keyblocktmp);
			keyblocklength /= 8;
			keyblock = malloc(keyblocklength);
			keyblocktmp = malloc(keyblocklength);
		} else {
		    printf("Invalid keyword: %s\n", ibuf);
		    err = 1;
		}
		if (keyblocklength && premasterlength)
			++step;
		break;
		}
	    else if (strncasecmp(ibuf,"COUNT = ", 8) != 0)
		{
		err = 1;
		printf("Missing new section or COUNT =  keyword\n");
		break;
		}
	    else
		step = 2;

	case 2: /* COUNT = and pre_master_secret = xxxx */
	    fputs(ibuf, rfp);
	    if(*ibuf == '\n')
		break;
	    if(!strncasecmp(ibuf,"COUNT = ",8))
		break;

	    if (strncasecmp(ibuf, "pre_master_secret = ", 20) != 0) {
		printf("Missing pre_master_secret\n");
		err = 1;
	    } else {
		len = hex2bin((char*)ibuf+20, premastersecret);
		if (len < 0) {
		    printf("Invalid premastersecret\n");
		    err =1;
		    break;
		}
                if (len != premasterlength) {
		    printf("Invalid len %d vs premasterlenght %d\n", len, premasterlength);
		    err =1;
		    break;
                }
		PrintValue("pre_master_secret", premastersecret, len);
		step++;
	    }
	    break;

	case 3: /* serverHello_random= xxxx */
	    fputs(ibuf, rfp);
	    if (strncasecmp(ibuf, "serverHello_random = ", 21) != 0) {
		printf("Missing serverHello_random\n");
		err = 1;
	    } else {
		len = hex2bin((char*)ibuf+21, serverHello_random);
		if (len < 0 || len > sizeof(serverHello_random)) {
		    printf("Invalid serverHello_random %d vs %d\n", len, (int)sizeof(serverHello_random));
		    err =1;
		    break;
		}
		PrintValue("serverHello_random", serverHello_random, len);
		serverhellorandomlen = len;
		step++;
	    }
	    break;

	case 4: /* clientHello_random = xxxx */
	    fputs(ibuf, rfp);
	    if (strncasecmp(ibuf, "clientHello_random = ", 21) != 0) {
		printf("Missing clientHello_random\n");
		err = 1;
	    } else {
		len = hex2bin((char*)ibuf+21, clientHello_random);
		if (len < 0 || len > sizeof(clientHello_random)) {
		    printf("Invalid clientHello_random %d vs %d\n", len, (int)sizeof(clientHello_random));
		    err =1;
		    break;
		}
		clienthellorandomlen = len;
		PrintValue("serverHello_random", clientHello_random, len);
		step++;
            }
	    break;

	case 5: /* server_random = xxxx */
	    fputs(ibuf, rfp);
	    if (strncasecmp(ibuf, "server_random = ", 16) != 0) {
		printf("Missing server_random\n");
		err = 1;
	    } else {
		len = hex2bin((char*)ibuf+16, server_random);
		if (len < 0 || len > sizeof(server_random)) {
		    printf("Invalid server_random %d vs %d\n", len, (int)sizeof(server_random));
		    err =1;
		    break;
		}
		serverrandomlen = len;
		PrintValue("server_random", server_random, len);
		step++;
            }
	    break;

	case 6: /* client_random = xxxx */
	    fputs(ibuf, rfp);
	    if (strncasecmp(ibuf, "client_random = ", 16) != 0) {
		printf("Missing client_random\n");
		err = 1;
		break;
	    }
	    len = hex2bin((char*)ibuf+16, client_random);
	    if (len < 0 || len > sizeof(client_random)) {
		printf("Invalid client_random %d vs %d\n", len, (int)sizeof(client_random));
		err =1;
		break;
	    }
	    PrintValue("client_random", client_random, len);
	    clientrandomlen = len;

	    /* shit->hit(fan) */

	    if ((tls != 10) && (tls != 12)) {
		printf("bad tls version %d\n", tls);
		err = 1;
		break;
	    }

	    if (tls == 10)
		digest_mask = TLS1_PRF;

	    if (tls == 12) {
		switch (shabits) {
		case 1:	digest_mask = TLS1_PRF; break;
		case 256:digest_mask = TLS1_PRF_SHA256; break;
		case 384:digest_mask = TLS1_PRF_SHA384; break;
		/*case 512:	digest_mask = TLS1_PRF_SHA512; break; not in openssl */
		default: printf("bad shabits %d\n", shabits);
			err=1;
			break;
		}
	    }
	    memcpy(seed, "master secret",strlen("master secret"));
	    memcpy(seed+strlen("master secret"), clientHello_random, clienthellorandomlen);
	    memcpy(seed+strlen("master secret")+clienthellorandomlen, serverHello_random, serverhellorandomlen);

	    private_tls1_PRF(digest_mask,
		    seed, strlen("master secret")+clienthellorandomlen+serverhellorandomlen,
		    NULL,	0,
		    NULL,	0,
		    NULL, 	0,
		    NULL,	0,
		    premastersecret, premasterlength,
		    mastersecret,tmpsecret,sizeof(tmpsecret));
	    OutputValue("master_secret", mastersecret, premasterlength, rfp, 0);

	    private_tls1_PRF(digest_mask,
		    "key expansion", strlen("key expansion"),
		    server_random, serverrandomlen,
		    client_random, clientrandomlen,
		    NULL, 0, NULL, 0,
		    mastersecret, premasterlength,
		    keyblock, keyblocktmp, keyblocklength
	    );
	    OutputValue("key_block", keyblock, keyblocklength, rfp, 0);

	    step++;
	    break;

	case 7:
	    if (ibuf[0] != '\n')
		{
		err = 1;
		printf("Missing terminator\n");
		}
	    fputs(ibuf, rfp);
	    step = 1;
	    break;
	    }
	}
    if (rfp)
	fclose(rfp);
    if (afp)
	fclose(afp);
    return err;
    }

/*--------------------------------------------------
  Processes either a single file or
  a set of files whose names are passed in a file.
  A single file is specified as:
    aes_test -f xxx.req
  A set of files is specified as:
    aes_test -d xxxxx.xxx
  The default is: -d req.txt
--------------------------------------------------*/
int main(int argc, char **argv)
    {
    char *rqlist = "req.txt", *rspfile = NULL;
    FILE *fp = NULL;
    char fn[250] = "", rfn[256] = "";
    int d_opt = 1;

#ifdef OPENSSL_FIPS
    if(!FIPS_mode_set(1))
	{
	do_print_errors();
	EXIT(1);
	}
#endif
    SSL_library_init();
    if (argc > 1)
	{
	if (strcasecmp(argv[1], "-d") == 0)
	    {
	    d_opt = 1;
	    }
	else if (strcasecmp(argv[1], "-f") == 0)
	    {
	    d_opt = 0;
	    }
	else
	    {
	    printf("Invalid parameter: %s\n", argv[1]);
	    return 0;
	    }
	if (argc < 3)
	    {
	    printf("Missing parameter\n");
	    return 0;
	    }
	if (d_opt)
	    rqlist = argv[2];
	else
	    {
	    strcpy(fn, argv[2]);
	    rspfile = argv[3];
	    }
	}
    if (d_opt)
	{ /* list of files (directory) */
	if (!(fp = fopen(rqlist, "r")))
	    {
	    printf("Cannot open req list file\n");
	    return -1;
	    }
	while (fgets(fn, sizeof(fn), fp))
	    {
	    strtok(fn, "\r\n");
	    strcpy(rfn, fn);
	    if (VERBOSE)
		printf("Processing: %s\n", rfn);
	    if (proc_file(rfn, rspfile))
		{
		printf(">>> Processing failed for: %s <<<\n", rfn);
		EXIT(1);
		}
	    }
	fclose(fp);
	}
    else /* single file */
	{
	if (VERBOSE)
	    printf("Processing: %s\n", fn);
	if (proc_file(fn, rspfile))
	    {
	    printf(">>> Processing failed for: %s <<<\n", fn);
	    }
	}
    EXIT(0);
    return 0;
    }

#endif

